@@ -15,26 +15,27 @@ fast.table <- function (data)
     bin <- bin[!is.na(bin)]                                                                
     if (length(bin)) bin <- bin + 1                                                        
     y <- tabulate(bin, pd)                                                                 
-    result=list(index = bin, weights = y, data = data[ind,])                                                                                  
+    result=list(index = bin, weights = y, data = data[ind,])                                         
     result                                                                                 
 }                                                                                        
 
 
 phyDat.default <- function (data, levels = NULL, return.index = TRUE, contrast = NULL, 
-    ambiguity = "?", compress=TRUE, ...) 
+                            ambiguity = "?", compress=TRUE, ...) 
 {
     if (is.matrix(data)) 
         nam = row.names(data)
     else nam = names(data)
     if(is.null(nam))stop("data object must contain taxa names")
+    if(inherits(data, "character")) data <- as.matrix(data)
     if (inherits(data,"DNAbin")) 
         data = as.character(data)
     if (is.matrix(data)) 
         data = as.data.frame(t(data), stringsAsFactors = FALSE)
-# new 4.4.2016 bug fix (reported by Eli Levy Karin)     
-    if (is.vector(data) && !is.list(data))data = as.data.frame(t(data), stringsAsFactors = FALSE)
+    # new 4.4.2016 bug fix (reported by Eli Levy Karin)     
+    #    if (is.vector(data) && !is.list(data))data = as.data.frame(data, stringsAsFactors = FALSE)
     else data = as.data.frame(data, stringsAsFactors = FALSE)
-#    data = data.frame(as.matrix(data), stringsAsFactors = FALSE)    
+    #    data = data.frame(as.matrix(data), stringsAsFactors = FALSE)    
     
     
     if(length(data[[1]])==1) compress=FALSE 
@@ -70,14 +71,33 @@ phyDat.default <- function (data, levels = NULL, return.index = TRUE, contrast =
                 contrast = rbind(contrast, matrix(1, k, l))
         }
     }
+    
+    
+#    row.names(data) = as.character(1:p)
+#    data = na.omit(data)
+#    rn = as.numeric(rownames(data))
+    
+    d = dim(data)
     att = attributes(data) 
-    data = lapply(data, match, all.levels) # avoid unlist   
+    
+#    print(system.time(match(unlist(data), all.levels)))
+    
+    data = match(unlist(data), all.levels)
+    attr(data, "dim") = d
+    data = as.data.frame(data, stringsAsFactors=FALSE)
     attributes(data) = att
     
+    #    att = attributes(data) 
+    #    data = lapply(data, match, all.levels) # avoid unlist   
+    #    attributes(data) = att
+    
     row.names(data) = as.character(1:p)
     data = na.omit(data)
-    
     aaa = match(index, attr(data, "na.action"))
+    
+
+    if(!is.null(attr(data, "na.action"))) warning("Found unknown characters (not supplied in levels). Deleted sites with with unknown states.")
+    
     index = index[is.na(aaa)] 
     index = match(index, unique(index))
     rn = as.numeric(rownames(data))
@@ -100,6 +120,7 @@ phyDat.default <- function (data, levels = NULL, return.index = TRUE, contrast =
 }
 
 
+
 phyDat.DNA = function (data, return.index = TRUE) 
 {
     if (is.matrix(data)) 
@@ -107,6 +128,7 @@ phyDat.DNA = function (data, return.index = TRUE)
     else nam = names(data)
     if (inherits(data,"DNAbin")) 
         data = as.character(data)
+    if(inherits(data, "character")) data <- as.matrix(data)
     if (is.matrix(data)) 
         data = as.data.frame(t(data), stringsAsFactors = FALSE)
     else data = as.data.frame(data, stringsAsFactors = FALSE)
@@ -122,9 +144,22 @@ phyDat.DNA = function (data, return.index = TRUE)
         0, 1, 1, 1, 1, 1, 1)), 18, 4, dimnames = list(NULL, c("a", 
         "c", "g", "t")))
     
-    ddd = fast.table(data)
-    data = ddd$data
-    index = ddd$index
+#    ddd = fast.table(data)
+    compress=TRUE
+    if(length(data[[1]])==1) compress=FALSE 
+    if(compress){
+        ddd = fast.table(data)
+        data = ddd$data
+        weight = ddd$weight
+        index = ddd$index
+    }
+    else{
+        p = length(data[[1]])
+        weight = rep(1, p)
+        index = 1:p
+    }
+#    data = ddd$data
+#    index = ddd$index
     q = length(data)
     p = length(data[[1]])
     d = dim(data)
@@ -137,14 +172,17 @@ phyDat.DNA = function (data, return.index = TRUE)
     row.names(data) = as.character(1:p)
     data = na.omit(data)
     rn = as.numeric(rownames(data))
-
+    
+    if(!is.null(attr(data, "na.action"))) warning("Found unknown characters. Deleted sites with with unknown states.")
+    
     aaa = match(index, attr(data, "na.action"))
     index = index[is.na(aaa)] 
     index = match(index, unique(index))
     rn = as.numeric(rownames(data))
     attr(data, "na.action") = NULL
     
-    weight = ddd$weight[rn]
+    weight = weight[rn]
+#    weight = ddd$weight[rn]
     p = dim(data)[1]
     names(data) = nam
     attr(data, "row.names") = NULL 
@@ -168,6 +206,7 @@ phyDat.AA <- function (data, return.index = TRUE)
     else nam = names(data)  
     if (inherits(data,"DNAbin")) 
         data = as.character(data)
+    if(inherits(data, "character")) data <- as.matrix(data)
     if (is.matrix(data)) 
         data = as.data.frame(t(data), stringsAsFactors = FALSE)
     else data = as.data.frame(data, stringsAsFactors = FALSE)
@@ -186,9 +225,22 @@ phyDat.AA <- function (data, return.index = TRUE)
     AA[23:25, ] = 1
     dimnames(AA) <- list(aa2, aa)
     
-    ddd = fast.table(data)
-    data = ddd$data
-    index = ddd$index
+#    ddd = fast.table(data)
+#    data = ddd$data
+#    index = ddd$index
+    compress=TRUE
+    if(length(data[[1]])==1) compress=FALSE 
+    if(compress){
+        ddd = fast.table(data)
+        data = ddd$data
+        weight = ddd$weight
+        index = ddd$index
+        }
+    else{
+        p = length(data[[1]])
+        weight = rep(1, p)
+        index = 1:p
+    }
     q = length(data)
     p = length(data[[1]])
     tmp <- vector("list", q)
@@ -204,13 +256,16 @@ phyDat.AA <- function (data, return.index = TRUE)
     data = na.omit(data)
     rn = as.numeric(rownames(data))
 
+    if(!is.null(attr(data, "na.action"))) warning("Found unknown characters. Deleted sites with with unknown states.")
+    
     aaa = match(index, attr(data, "na.action"))
     index = index[is.na(aaa)] 
     index = match(index, unique(index))
     rn = as.numeric(rownames(data))
     attr(data, "na.action") = NULL
-        
-    weight = ddd$weight[rn]
+  
+#    weight = ddd$weight[rn]
+    weight = weight[rn]  
     p = dim(data)[1]
     names(data) = nam
     attr(data, "row.names") = NULL
@@ -235,7 +290,7 @@ phyDat.codon <- function (data, return.index = TRUE)
     else nam = names(data)  
     if (inherits(data,"DNAbin")) 
         data = as.character(data)
-
+    if(inherits(data, "character")) data <- as.matrix(data)
     if (is.matrix(data)) 
         data = as.data.frame(t(data), stringsAsFactors = FALSE)
     else data = as.data.frame(data, stringsAsFactors = FALSE)
@@ -250,9 +305,21 @@ phyDat.codon <- function (data, return.index = TRUE)
         sapply(starts, function(x) paste(seq[x:(x + 2L)], collapse=""))
     } 
  
-    data = sapply(data, splseq)
-    
-    ddd = fast.table(data)
+    data = data.frame(lapply(data, splseq))
+#    ddd = fast.table(data)
+    compress=TRUE
+    if(nrow(data)==1) compress=FALSE 
+    if(compress){
+            ddd = fast.table(data)
+            data = ddd$data
+            weight = ddd$weight
+            index = ddd$index
+        }
+    else{
+        p = length(data[[1]])
+        weight = rep(1, p)
+        index = 1:p
+    }
     codon = c("aaa", "aac", "aag", "aat", "aca", "acc", "acg", "act", 
       "aga", "agc", "agg", "agt", "ata", "atc", "atg", "att", 
       "caa", "cac", "cag", "cat", "cca", "ccc", "ccg", "cct", "cga", 
@@ -266,8 +333,8 @@ phyDat.codon <- function (data, return.index = TRUE)
     CODON <- diag(61)
     dimnames(CODON) <- list(codon, codon)
 
-    data = ddd$data
-    index = ddd$index
+#    data = ddd$data
+#    index = ddd$index
     q = length(data)
     p = length(data[[1]])
     tmp <- vector("list", q)
@@ -282,14 +349,17 @@ phyDat.codon <- function (data, return.index = TRUE)
     row.names(data) = as.character(1:p)
     data = na.omit(data)
     rn = as.numeric(rownames(data))
-
+    
+    if(!is.null(attr(data, "na.action"))) warning("Found unknown characters. Deleted sites with with unknown states.")
+    
     aaa = match(index, attr(data, "na.action"))
     index = index[is.na(aaa)] 
     index = match(index, unique(index))
     rn = as.numeric(rownames(data))
     attr(data, "na.action") = NULL
         
-    weight = ddd$weight[rn]
+#    weight = ddd$weight[rn]
+    weight = weight[rn]
     p = dim(data)[1]
     names(data) = nam
     attr(data, "row.names") = NULL
@@ -307,17 +377,153 @@ phyDat.codon <- function (data, return.index = TRUE)
 }
 
 
+#' Conversion among Sequence Formats
+#' 
+#' These functions transform several DNA formats into the \code{phyDat} format.
+#' \code{allSitePattern} generates an alignment of all possible site patterns.
+#' 
+#' If \code{type} "USER" a vector has to be give to \code{levels}. For example
+#' c("a", "c", "g", "t", "-") would create a data object that can be used in
+#' phylogenetic analysis with gaps as fifth state.  There is a more detailed
+#' example for specifying "USER" defined data formats in the vignette
+#' "phangorn-specials".
+#' 
+#' \code{allSitePattern} returns all possible site patterns and can be useful
+#' in simulation studies. For further details see the vignette
+#' phangorn-specials.
+#' 
+#' \code{write.phyDat} calls the function write.dna or write.nexus.data and
+#' \code{read.phyDat} calls the function \code{read.dna}, \code{read.aa} or
+#' \code{read.nexus.data} see for more details over there.
+#' 
+#' You may import data directly with \code{\link[ape]{read.dna}} or
+#' \code{\link[ape]{read.nexus.data}} and convert the data to class phyDat.
+#' 
+#' The generic function \code{c} can be used to to combine sequences and
+#' \code{unique} to get all unique sequences or unique haplotypes.
+#' 
+#' \code{acgt2ry} converts a \code{phyDat} object of nucleotides into an binary
+#' ry-coded dataset.
+#' 
+#' @aliases 
+#' as.phyDat.character as.phyDat.data.frame as.phyDat.matrix
+#' as.MultipleAlignment as.MultipleAlignment.phyDat cbind.phyDat c.phyDat 
+#' acgt2ry removeUndeterminedSites phyDat2MultipleAlignment 
+#' @param data An object containing sequences.
+#' @param x An object containing sequences.
+#' @param type Type of sequences ("DNA", "AA", "CODON" or "USER").
+#' @param levels Level attributes.
+#' @param return.index If TRUE returns a index of the site patterns.
+#' @param file A file name.
+#' @param format File format of the sequence alignment (see details).  Several
+#' popular formats are supported: "phylip", "interleaved", "sequential",
+#' "clustal", "fasta" or "nexus", or any unambiguous abbreviation of these.
+#' @param colsep a character used to separate the columns (a single space by
+#' default).
+#' @param nbcol a numeric specifying the number of columns per row (-1 by
+#' default); may be negative implying that the nucleotides are printed on a
+#' single line.
+#' @param n Number of sequences.
+#' @param names Names of sequences.
+#' @param subset a subset of taxa.
+#' @param select a subset of characters.
+#' @param site.pattern select site pattern or sites.
+#' @param allLevels return original data.
+#' @param obj as object of class phyDat
+#' @param freq logical, if 'TRUE', frequencies or counts are returned otherwise
+#' proportions
+#' @param all all a logical; if all = TRUE, all counts of bases, ambiguous
+#' codes, missing data, and alignment gaps are returned as defined in the
+#' contrast.
+#' @param drop.unused.levels logical, drop unused levels
+#' @param incomparables for compatibility with unique.
+#' @param identical if TRUE (default) sequences have to be identical, if FALSE
+#' sequences are considered duplicates if distance between sequences is zero
+#' (happens frequently with ambiguous sites).
+#' @param ambiguity character for ambiguous character and no contrast is provided.
+#' @param ... further arguments passed to or from other methods.
+#' @return The functions return an object of class \code{phyDat}.
+#' @author Klaus Schliep \email{klaus.schliep@@gmail.com}
+#' @seealso \code{\link{DNAbin}}, \code{\link{as.DNAbin}},
+#' \code{\link{read.dna}}, \code{\link{read.aa}}, \code{\link{read.nexus.data}}
+#' and the chapter 1 in the \code{vignette("phangorn-specials",
+#' package="phangorn")} and the example of \code{\link{pmlMix}} for the use of
+#' \code{allSitePattern}
+#' @keywords cluster
+#' @examples
+#' 
+#' data(Laurasiatherian)
+#' class(Laurasiatherian)
+#' Laurasiatherian
+#' baseFreq(Laurasiatherian)
+#' baseFreq(Laurasiatherian, all=TRUE)
+#' subset(Laurasiatherian, subset=1:5)
+#' # transform into old ape format
+#' LauraChar <- as.character(Laurasiatherian)
+#' # and back 
+#' Laura <- phyDat(LauraChar, return.index=TRUE)
+#' all.equal(Laurasiatherian, Laura)
+#' allSitePattern(5)
+#' 
+#' @rdname phyDat
+#' @export 
+phyDat <- function (data, type="DNA", levels=NULL, return.index = TRUE,...) 
+{
+    if (inherits(data,"DNAbin")) type <- "DNA"
+    pt <- match.arg(type, c("DNA", "AA", "CODON", "USER"))  
+    if(pt=="DNA") dat <- phyDat.DNA(data, return.index=return.index,...)
+    if(pt=="AA") dat <- phyDat.AA(data, return.index=return.index, ...)
+    if(pt=="CODON") dat <- phyDat.codon(data, return.index=return.index, ...)
+    if(pt=="USER") dat <- phyDat.default(data, levels = levels, return.index=return.index, ...)
+    dat
+}
+
+
+#' @rdname phyDat
+#' @export
+dna2codon <- function(x){
+    if(!inherits(x, "phyDat"))stop("x needs to be of class phyDat!")
+    phyDat.codon(as.character(x))
+}
+
+
+#' @rdname phyDat
+#' @export
+codon2dna <- function(x){
+    if(!inherits(x, "phyDat"))stop("x needs to be of class phyDat!")
+    phyDat.DNA(as.character(x))
+}
+
+
+#' @rdname phyDat
+#' @export
 as.phyDat <- function (x, ...){
     if (inherits(x,"phyDat")) return(x)
     UseMethod("as.phyDat")
 }
 
 
+#' @rdname phyDat
+#' @method as.phyDat factor
+#' @export
+as.phyDat.factor <- function(x, ...){
+    nam <- names(x)
+    lev <- levels(x)
+    x <- as.character(x)
+    names(x) <- nam
+    phyDat(x, type="USER", levels = lev, ...)
+}
+
+
+#' @rdname phyDat
+#' @method as.phyDat DNAbin
+#' @export
 as.phyDat.DNAbin <- function(x,...) phyDat.DNA(x,...)
 
 
-
-
+#' @rdname phyDat
+#' @method as.phyDat alignment
+#' @export
 as.phyDat.alignment <- function (x, type="DNA",...) 
 {
     x$seq <- tolower(x$seq)
@@ -332,7 +538,8 @@ as.phyDat.alignment <- function (x, type="DNA",...)
 
 
 #as.alignment.phyDat <- function(x, ...) as.alignment(as.character(x))
-
+#' @rdname phyDat
+#' @export
 phyDat2alignment <-  function(x){
     z = as.character(x)
     nam = rownames(z)
@@ -347,6 +554,9 @@ phyDat2alignment <-  function(x){
 }
 
 
+#' @rdname phyDat
+#' @method as.phyDat MultipleAlignment
+#' @export
 as.phyDat.MultipleAlignment <- function(x, ...){
     if (requireNamespace('Biostrings')){
     if(inherits(x, "DNAMultipleAlignment"))
@@ -367,7 +577,7 @@ as.MultipleAlignment <- function (x, ...){
 }
 
 
-as.MultipleAlignment.phyDat <- function(x){
+as.MultipleAlignment.phyDat <- function(x, ...){
     if (requireNamespace('Biostrings')){
     z = as.character(x)
     nam = rownames(z)
@@ -388,6 +598,9 @@ phyDat2MultipleAlignment <- as.MultipleAlignment.phyDat
 as.phyDat.matrix <- function (x, ...) phyDat(data=x, ...)
 
 
+as.phyDat.character <- function (x, ...) phyDat(data=x, ...)
+
+
 as.phyDat.data.frame <- function (x, ...) phyDat(data=x, ...)
  
 
@@ -413,24 +626,25 @@ acgt2ry <- function(obj){
 }
 
 
+#' @rdname phyDat
+#' @export
+# replace as.character.phyDat weniger Zeilen, works also for codons
 as.character.phyDat <- function (x, allLevels=TRUE, ...) 
 {
     nr <- attr(x, "nr")
     nc <- attr(x, "nc")
     type <- attr(x, "type")
-    if (type == "DNA") {
-        labels <- c("a", "c", "g", "t", "u", "m", "r", "w", "s", 
-            "y", "k", "v", "h", "d", "b", "n", "?", "-")
-    }
-    if (type == "AA") {
-        labels <- c("a", "r", "n", "d", "c", "q", "e", "g", "h", 
-            "i", "l", "k", "m", "f", "p", "s", "t", "w", "y", 
-            "v", "b", "z", "x", "-", "?")
-    }
+    labels = attr(x, "allLevels")
+    
+    if (!is.null(attr(x, "index"))) {
+        index = attr(x, "index")
+        if (is.data.frame(index)) 
+            index <- index[, 1]
+    } 
+    else index = rep(1:nr, attr(x, "weight"))
     if (type == "USER") {
-        #levels
-        if(allLevels)labels = attr(x, "allLevels")
-        else{
+        #levels in acgt2ry
+        if(!allLevels){
             tmp = attr(x, "levels")
             contrast = attr(x, "contrast") # contrast=AC
             contrast[contrast>0] = 1
@@ -438,70 +652,32 @@ as.character.phyDat <- function (x, allLevels=TRUE, ...)
             contrast[rowSums(contrast)>1,] = 0 
             labels = rep(NA, length(attr(x, "allLevels")))
             labels[ind] = tmp[contrast%*%c(1:length(tmp))]
-            }
+        }
+    }
+    if(type == "AA") labels <- toupper(labels)
+    if(type == "CODON"){
+        nr <- length(index)
+        result = matrix(NA, nrow = length(x), ncol = 3L*nr)
+        labels = strsplit(labels, "")
+        for (i in 1:length(x)) result[i, ] <- unlist(labels[ x[[i]][index] ])
     }
-    result = matrix(NA, nrow = length(x), ncol = nr)
-    for (i in 1:length(x)) result[i, ] <- labels[x[[i]]]
-    if (is.null(attr(x, "index"))) 
-        index = rep(1:nr, attr(x, "weight"))
     else {
-        index = attr(x, "index")
-        if (is.data.frame(index)) 
-            index <- index[, 1]
+        result = matrix(NA, nrow = length(x), ncol = nr)
+        for (i in 1:length(x)) result[i, ] <- labels[x[[i]]]
+        result = result[, index, drop = FALSE]
     }
-    result = result[, index, drop = FALSE]
     rownames(result) = names(x)
     result
 }
 
 
-# replace as.character.phyDat 20 Zeilen weniger
-as.character.phyDat2 <- function (x, ...) 
-{
-    nr <- attr(x, "nr")
-    nc <- attr(x, "nc")
-    type <- attr(x, "type")
-    labels = attr(x, "allLevels")
-    result = matrix(NA, nrow = length(x), ncol = nr)
-    for (i in 1:length(x)) result[i, ] <- labels[x[[i]]]
-    if (is.null(attr(x, "index"))) 
-        index = rep(1:nr, attr(x, "weight"))
-    else {
-        index = attr(x, "index")
-        if (is.data.frame(index)) 
-            index <- index[, 1]
-    }
-    result = result[, index, drop = FALSE]
-    rownames(result) = names(x)
-    result
-}
-
-
-#as.data.frame.phyDat <- function(x, ...){
-#   data.frame(t(as.character(x, ...)), stringsAsFactors=FALSE)
-#}
-
-# much faster
-# TODO as stringsAsFactors=FALSE
-# result[[i]] <- x[[i]] + factor levels setzen
-# 
-as.data.frame.phyDatOld <- function(x, ...){
-    nr <- attr(x, "nr")
-    nc <- attr(x, "nc")
-    labels <- attr(x, "allLevels")
-    result <- vector("list", length(x))
-    for (i in 1:length(x)) result[[i]] <- labels[x[[i]]]
-    attr(result, "names") <- names(x)
-    attr(result, "row.names") <- 1:nr
-    attr(result, "class") <- "data.frame"
-    result
-}
-
-
+#' @rdname phyDat
+#' @export
 as.data.frame.phyDat <- function(x, ...){
   nr <- attr(x, "nr")
   nc <- attr(x, "nc")
   labels <- attr(x, "allLevels")
+  if(attr(x, "type") == "AA") labels <- toupper(labels)
   result <- vector("list", length(x))
   if (is.null(attr(x, "index"))) 
     index = rep(1:nr, attr(x, "weight"))
@@ -524,6 +700,8 @@ as.data.frame.phyDat <- function(x, ...){
 #}
 
 # quite abit faster
+#' @rdname phyDat
+#' @export
 as.DNAbin.phyDat <- function (x, ...) 
 {
     if(attr(x, "type")=="DNA"){
@@ -560,24 +738,13 @@ as.DNAbin.phyDat <- function (x, ...)
 }
 
 
-if (getRversion() >= "2.15.1") utils::globalVariables("as.AAbin")
+#' @rdname phyDat
+#' @export
 as.AAbin.phyDat <- function(x,...) {
    if(attr(x, "type")=="AA") return(as.AAbin(as.character(x, ...)))
    else stop("x must be a amino acid sequence")
 }
 
- 
-phyDat <- function (data, type="DNA", levels=NULL, return.index = TRUE,...) 
-{
-    if (inherits(data,"DNAbin")) type <- "DNA"
-    pt <- match.arg(type, c("DNA", "AA", "CODON", "USER"))  
-    if(pt=="DNA") dat <- phyDat.DNA(data, return.index=return.index,...)
-    if(pt=="AA") dat <- phyDat.AA(data, return.index=return.index, ...)
-    if(pt=="CODON") dat <- phyDat.codon(data, return.index=return.index, ...)
-    if(pt=="USER") dat <- phyDat.default(data, levels = levels, return.index=return.index, ...)
-    dat
-}
-
 
 print.phyDat = function (x, ...) 
 {
@@ -586,51 +753,42 @@ print.phyDat = function (x, ...)
 }
 
 
-c.phyDat <- function(...){
-    object <- as.list(substitute(list(...)))[-1]    
-    x <- list(...)
-    n <- length(x) 
-    match.names <- function(a,b){
-        if(any(!(a %in% b)))stop("names do not match previous names") 
-        }
-    if (n == 1) 
-        return(x[[1]])
-    type <- attr(x[[1]], "type")
-    nr = numeric(n)
-    nr[1] <- sum(attr(x[[1]], "weight"))
-    levels <- attr(x[[1]], "levels")
-    snames <- names(x[[1]])
-    objNames<-as.character(object)
-    if(any(duplicated(objNames))) objNames <- paste0(objNames, 1:n)
-    tmp <- as.character(x[[1]])
-    for(i in 2:n){
-        match.names(snames,names(x[[i]]))
-        x[[i]] <- getCols(x[[i]],snames)
-        nr[i] <- sum(attr(x[[i]], "weight"))
-        tmp <- cbind(tmp, as.character(x[[i]]))
-    }
-    if (type == "DNA") 
-        dat <- phyDat.DNA(tmp, return.index = TRUE)
-    if (type == "AA") 
-        dat <- phyDat.AA(tmp, return.index = TRUE)
-    if (type == "USER") 
-        dat <- phyDat.default(tmp, levels = levels, return.index = TRUE)
-     if (type == "CODON") 
-        dat <- phyDat.codon(tmp, return.index = TRUE)       
-    attr(dat,"index") <- data.frame(index=attr(dat,"index"), genes=rep(objNames, nr))   
-    dat
+# in C++ to replace aggregate or use of distinct from dplyr / data.table
+aggr <- function(weight, ind){
+    res <- numeric(max(ind))
+    for(i in seq_along(weight))
+        res[ind[i]] = res[ind[i]] + weight[i]
+    res
 }
 
 
+# data has to be a data.frame in cbind.phyDat
+fast.table2 <- function (data)                                                            
+{                                                                                 
+    if(!is.data.frame(data)) 
+        data <- as.data.frame(data, stringsAsFactors = FALSE)                    
+    da <- do.call("paste", data)                                             
+    ind <- !duplicated(da)                                                                  
+    levels <- da[ind]                                                                       
+    cat <- factor(da,levels = levels)                                                      
+    nl <- length(levels)                                                       
+    bin <- (as.integer(cat) - 1L)                                                           
+    bin <- bin[!is.na(bin)]                                                                
+    if (length(bin)) bin <- bin + 1L                                                        
+    result=list(index = bin, pos = ind)                                                                    
+    result                                                                                 
+}                                                                                        
+
+
 # new cbind.phyDat 
-cbindPD <- function(..., gaps="-"){
+cbind.phyDat <- function(..., gaps="-", compress=TRUE){
     object <- as.list(substitute(list(...)))[-1]    
     x <- list(...)
     n <- length(x) 
     if (n == 1) 
         return(x[[1]])
     type <- attr(x[[1]], "type")
-    nr = numeric(n)
+    nr <- numeric(n)
     
     ATTR <- attributes(x[[1]])
     
@@ -639,98 +797,79 @@ cbindPD <- function(..., gaps="-"){
     allLevels <- attr(x[[1]], "allLevels")
     gapsInd <- match(gaps, allLevels)
     snames <- vector("list", n)  # names(x[[1]])
-    vec = numeric(n+1)
-    wvec = numeric(n+1)
-    objNames<-as.character(object)
+    vec <- numeric(n+1)
+    wvec <- numeric(n+1)
+    objNames <- as.character(object)
     if(any(duplicated(objNames))) objNames <- paste0(objNames, 1:n)
     #    tmp <- as.character(x[[1]])
     
     for(i in 1:n){
-        snames[[i]] = names(x[[i]]) 
-        nr[i] <- attr(x[[i]], "nr") 
-        vec[i+1] = attr(x[[i]], "nr")
-        wvec[i+1] = sum(attr(x[[i]], "weight"))
+        snames[[i]] <- names(x[[i]]) 
+        nr[i] <- sum(attr(x[[i]], "weight")) 
+        vec[i+1] <- attr(x[[i]], "nr")
+        wvec[i+1] <- sum(attr(x[[i]], "weight"))
     }
-    vec = cumsum(vec)
-    wvec = cumsum(wvec)
-    snames = unique(unlist(snames))
+    vec <- cumsum(vec)
+    wvec <- cumsum(wvec)
+    snames <- unique(unlist(snames))
     weight <- numeric(vec[n+1])
     
-    index <- numeric(wvec[n+1]) 
+    index <- numeric(wvec[n+1])
+    
     ATTR$names <- snames
     ATTR$nr <- vec[n+1]
     
-    tmp = matrix(gapsInd, vec[n+1], length(snames), dimnames = list(NULL, snames))
+    tmp <- matrix(gapsInd, vec[n+1], length(snames), dimnames = list(NULL, snames))
     tmp <- as.data.frame(tmp)
-    
+    add.index <- TRUE
     for(i in 1:n){
-        nam = names(x[[i]])
+        nam <- names(x[[i]])
         tmp[(vec[i]+1):vec[i+1], nam] <- x[[i]][nam]
         weight[(vec[i]+1):vec[i+1]] <- attr(x[[i]], "weight")
-        index[(wvec[i]+1):wvec[i+1]] <- attr(x[[i]], "index")
     }
-    ATTR$index <- index
+    if(compress){
+        ddd <- fast.table2(tmp)
+        tmp <- tmp[ddd$pos,]
+        weight <- aggregate(weight, by=list(ddd$index), FUN=sum)$x
+    }
+    for(i in 1:n){
+        tmp2 <- attr(x[[i]], "index")
+        if(!is.null(tmp2)){
+            if(is.data.frame(tmp2))index[(wvec[i]+1):wvec[i+1]] <- ddd$index[(vec[i]+1):vec[i+1]][tmp2[,1]]
+            else index[(wvec[i]+1):wvec[i+1]] <- ddd$index[(vec[i]+1):vec[i+1]][tmp2]           
+        }
+        else add.index <- FALSE
+    }
+    if(add.index)ATTR$index <- data.frame(index = index, genes=rep(objNames, nr))
     ATTR$weight <- weight
+    ATTR$nr <- length(weight)
     attributes(tmp) <- ATTR
     tmp
 }
 
 
+c.phyDat <- cbind.phyDat
 
-cbind.phyDat <- function(..., gaps="-"){
-    object <- as.list(substitute(list(...)))[-1]    
-    x <- list(...)
-    n <- length(x) 
-    if (n == 1) 
-        return(x[[1]])
-    type <- attr(x[[1]], "type")
-    nr = numeric(n)
-    nr[1] <- sum(attr(x[[1]], "weight"))
-    levels <- attr(x[[1]], "levels")
-    snames <- vector("list", n)  # names(x[[1]])
-    vec = numeric(n+1)
-    objNames<-as.character(object)
-    if(any(duplicated(objNames))) objNames <- paste0(objNames, 1:n)
-    tmp <- as.character(x[[1]])
-    for(i in 1:n){
-        snames[[i]] = names(x[[i]]) #match.names(snames,names(x[[i]]))
-        nr[i] <- sum(attr(x[[i]], "weight")) 
-        vec[i+1] = sum(attr(x[[i]], "weight"))
+
+#' @rdname phyDat
+#' @export
+write.phyDat <- function(x, file, format="phylip", colsep = "", nbcol=-1, ...){
+    formats <- c("phylip", "nexus", "interleaved", "sequential", "fasta")
+    format <- match.arg(tolower(format), formats)
+    if(format=="nexus"){   
+        type = attr(x, "type")
+        if(type=="DNA") write.nexus.data(as.list(as.data.frame(x)), file, format = "dna",...)
+        else write.nexus.data(as.list(as.data.frame(x)), file, format = "protein", ...)
     }
-    vec = cumsum(vec)
-    snames = unique(unlist(snames))
-
-    tmp = matrix(gaps, length(snames), vec[n+1], dimnames = list(snames, NULL))
-
-    for(i in 1:n){
-        nam = names(x[[i]])
-        tmp[nam,(vec[i]+1):vec[i+1] ] <- as.character(x[[i]])
-    }
-    if (type == "DNA") 
-        dat <- phyDat.DNA(tmp, return.index = TRUE)
-    if (type == "AA") 
-        dat <- phyDat.AA(tmp, return.index = TRUE)
-    if (type == "USER") 
-        dat <- phyDat.default(tmp, levels = levels, 
-            return.index = TRUE)
-    if (type == "CODON") 
-        dat <- phyDat.codon(tmp, return.index = TRUE)            
-    attr(dat,"index") <- data.frame(index=attr(dat,"index"), genes=rep(objNames, nr))   
-    dat
+    else{
+        if(format=="phylip") format = "interleaved" 
+        write.dna(as.character(x), file, format=format, colsep = colsep, nbcol=nbcol, ...)
+    }    
 }
 
 
-write.phyDat <- function(x, file, format="phylip",...){
-    if(format=="fasta") write.dna(as.character(x), file, format="fasta", colsep = "", nbcol=-1, ...)
-    if(format=="phylip") write.dna(as.character(x), file, format="sequential", ...)    
-    if(format=="nexus"){   
-         type = attr(x, "type")
-         if(type=="DNA") write.nexus.data(as.list(as.data.frame(x)), file, format = "dna",...)
-         else write.nexus.data(as.list(as.data.frame(x)), file, format = "protein", ...)
-         }
-    }
-
-
+#' @rdname phyDat
+#' @export
 read.phyDat <- function(file, format="phylip", type="DNA", ...){
     
     formats <- c("phylip", "nexus", "interleaved", "sequential", "fasta", "clustal")
@@ -743,12 +882,21 @@ read.phyDat <- function(file, format="phylip", type="DNA", ...){
             data = read.dna(file, format, as.character = TRUE, ...)
         }
         if (type == "AA") data = read.aa(file, format=format, ...)
+        if (type == "USER"){
+            data = read.dna(file, format, as.character = TRUE)
+            extras <- match.call(expand.dots = FALSE)$...
+            extras <- lapply(extras, eval)
+            return(phyDat(data, type, levels=extras$levels, ambiguity = extras$ambiguity, 
+                contrast = extras$contrast))
+        }
         # raus
     }
     phyDat(data, type, return.index = TRUE)
 }
 
 
+#' @rdname phyDat
+#' @export
 baseFreq <- function(obj, freq=FALSE, all=FALSE, drop.unused.levels = FALSE){
     if (!inherits(obj,"phyDat")) 
         stop("data must be of class phyDat")
@@ -810,6 +958,9 @@ getRows <- function (data, rows, site.pattern = TRUE)
 }
 
 
+#' @rdname phyDat
+#' @method subset phyDat
+#' @export
 subset.phyDat <- function (x, subset, select, site.pattern = TRUE,...) 
 {  
      
@@ -874,6 +1025,9 @@ duplicated_map <- function(x, ...){
 }   
 
 
+#' @rdname phyDat
+#' @method unique phyDat
+#' @export
 unique.phyDat <- function(x, incomparables=FALSE, identical=TRUE, ...){
     if(identical) return(getCols(x, !duplicated(x)))
     getCols(x, !duplicated_phyDat(x))
@@ -901,7 +1055,8 @@ removeParsUninfoSites <- function(data){
 }
 
 
-
+#' @rdname phyDat
+#' @export
 allSitePattern <- function(n,levels=c("a","c","g","t"), names=NULL){
     l=length(levels)
     X=vector("list", n)
@@ -935,16 +1090,29 @@ write.phylip <- function(data, weight, file=""){
 
 read.FASTA.AA <- function (file) 
 {
-    if (length(grep("^(ht|f)tp:", file))) {
+    if (length(grep("^(ht|f)tp(s|):", file))) {
         url <- file
         file <- tempfile()
         download.file(url, file)
     }
-    sz <- file.info(file)$size
-    x <- readBin(file, "raw", sz)
-    icr <- which(x == as.raw(13))
-    if (length(icr)) 
-        x <- x[-icr]
+    if (is(file, "connection")) {
+        if (!isOpen(file, "rt")) {
+            open(file, "rt")
+            on.exit(close(file))
+        }
+        x <- scan(file, what = character(), sep = "\n", quiet = TRUE)
+        x <- charToRaw(paste(x, collapse = "\n"))
+        sz <- length(x)
+    } else {
+        sz <- file.size(file)
+        x <- readBin(file, "raw", sz)
+    }
+    ## if the file is larger than 1 Gb we assume that it is
+    ## UNIX-encoded and skip the search-replace of carriage returns
+    if (sz < 1e9) {
+        icr <- which(x == as.raw(0x0d)) # CR
+        if (length(icr)) x <- x[-icr]
+    }
     res <- .Call("rawStream2phyDat", x)
     
     aa <- c("a", "r", "n", "d", "c", "q", "e", "g", "h", "i", 
@@ -980,6 +1148,39 @@ read.FASTA.AA <- function (file)
 
 
 # throw out
+
+
+#' Read Amino Acid Sequences in a File
+#' 
+#' This function reads amino acid sequences in a file, and returns a matrix
+#' list of DNA sequences with the names of the taxa read in the file as row
+#' names.
+#' 
+#' 
+#' @param file a file name specified by either a variable of mode character, or
+#' a double-quoted string.
+#' @param format a character string specifying the format of the DNA sequences.
+#' Three choices are possible: \code{"interleaved"}, \code{"sequential"}, or
+#' \code{"fasta"}, or any unambiguous abbreviation of these.
+#' @param skip the number of lines of the input file to skip before beginning
+#' to read data.
+#' @param nlines the number of lines to be read (by default the file is read
+#' until its end).
+#' @param comment.char a single character, the remaining of the line after this
+#' character is ignored.
+#' @param seq.names the names to give to each sequence; by default the names
+#' read in the file are used.
+#' @return a matrix of amino acid sequences.
+#' @author Klaus Schliep \email{klaus.schliep@@gmail.com}
+#' @seealso \code{\link[ape]{read.dna}}, \code{\link[ape]{read.GenBank}},
+#' \code{\link[phangorn]{phyDat}}, \code{\link[seqinr]{read.alignment}}
+#' @references % Anonymous. FASTA format description. %
+#' \url{https://www.ncbi.nlm.nih.gov/blast/fasta.shtml} Felsenstein, J. (1993)
+#' Phylip (Phylogeny Inference Package) version 3.5c. Department of Genetics,
+#' University of Washington.
+#' \url{http://evolution.genetics.washington.edu/phylip/phylip.html}
+#' @keywords IO
+#' @export read.aa
 read.aa <- function (file, format = "interleaved", skip = 0, nlines = 0, 
     comment.char = "#", seq.names = NULL) 
 {
@@ -1084,6 +1285,8 @@ read.aa <- function (file, format = "interleaved", skip = 0, nlines = 0,
 }
 
 
+#' @rdname phyDat
+#' @export
 genlight2phyDat <- function(x, ambiguity=NA){
     tmp <- as.matrix(x)
     lev <- na.omit(unique(as.vector(tmp)))
@@ -1091,7 +1294,9 @@ genlight2phyDat <- function(x, ambiguity=NA){
 }
 
 
-if (getRversion() >= "2.15.1") utils::globalVariables("image.AAbin")
+#' @rdname phyDat
+#' @method image phyDat
+#' @export
 image.phyDat <- function(x, ...){
     if(attr(x, "type")=="AA")image(as.AAbin(x), ...)
     if(attr(x, "type")=="DNA")image(as.DNAbin(x), ...)
